<?php
class ManageDomainAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('AJAX', 'validateAjax');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'manageDomain';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 20;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Domainliste");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL && $user->getIsAdmin() == 't')
      return true;
    return false;
  }

  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    if($this->request->getSubmitParameter('delete') != '')
    {
      $definitions = array(
          'delete' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                        'comparedb' => array('query' => 'SELECT COUNT(*) FROM domain WHERE domainname = {checkValue}',
                                            'compareType' => '==', 'compareValue' => 1)),
      );
    }
    $names = array(
          'delete' => _('Domain zu l&ouml;schen'),
    );
    $errorMsgs = array(
          'delete' => array(myMVC_BaseAction::$ERROR_COMPAREDB => _('Die Domain existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );

    if(!is_array($definitions))
      return true;
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $definitions = array(
        'newDomain' => array('type' => myMVC_BaseAction::$VALIDATE_WEB, 'empty' => false, 'justdomain' => true,
                                  'comparedb' => array('query' => 'SELECT COUNT(*) FROM domain WHERE domainname = {checkValue}',
                                      'compareType' => '==', 'compareValue' => 0)),
    );
    $names = array(
        'newDomain' => _('Neue Domain'),
    );
    $errorMsgs = array(
        'newDomain' => array(myMVC_BaseAction::$ERROR_COMPAREDB => _('Die Domain ist bereits registriert'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                             myMVC_BaseAction::$ERROR_NOTWEB    => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as AJAX.
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateAjax()
  {
    $definitions = array(
        'search'     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 2, 'maxlen' => 255, 'empty' => true),
    );
    $names = array(
        'search'     => _('Suchkriterium'),
    );
    $errorMsgs = array(
        'search'     => array(myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                              myMVC_BaseAction::$ERROR_MINLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}'),
                              myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('ManageDomainView');
    $this->session->useNamespace('manageDomain');
    
    if($this->request->getRequestMethod() == 'GET')
    {
      if($this->request->getSubmitParameter('delete') != '')
      {
        $domain = Domain::findByPK(array('domainname' => $this->request->getParameter('delete')));
        $domain->delete();
        $this->saveParameterForNextPage('popupMessage', _('Die Domain wurde erfolgreich gel&ouml;scht'));
      }
    }
    else
    {
      $domain = new Domain();
      $domain->setDomainname($this->request->getParameter('newDomain'));
      $domain->save();
      $this->saveParameterForNextPage('popupMessage', _('Die Domain wurde hinzugef&uuml;t und ist in K&uuml;rze nutzbar'));
    }
    $this->saveParameterForNextPage('domains', Domain::callFinder('findAll', array(), 'domainname ASC'));

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('ManageDomainView');
    $this->saveParameterForNextPage('error', true);
    $this->saveParameterForNextPage('domains', Domain::callFinder('findAll', array(), 'domainname ASC'));

    return $response;
  }

  /**
   * Perform the AJAX-Action
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function performAjax()
  {
    global $controller;

    $this->session->useNamespace('main');
    header('Content-Type: application/json');

    if($this->request->getParameter('search') != '')
      $domains = Domain::callFinder('findByDomainname', array('domainname' => $this->request->getParameter('search')), 'domainname ASC');
    else
      $domains = Domain::callFinder('findAll', array(), 'domainname ASC');

    $renderEngine = new myMVC_SmartyEngine(dirname(__FILE__).'/../pages/templates',
                                           dirname(__FILE__).'/../pages/templates_c');
    $view = new SearchView($renderEngine, $this->session, 'searchDomains.tpl');
    $view->setObject('domains', $domains);
    $controller->setCurrentLanguage();
    if($view->preRender())
      $html = $view->fetch();
    else
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SearchView::preRender() returned false. Unable to display page');
      throw new myMVC_MVCException($msg);
    }

    print json_encode(array('result' => 'OK', 'html' => $html));
    exit;
  }

  /**
   * Called if user data are not valid (check with validate)
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidDataAjax()
  {
    header('Content-Type: application/json');
    $errors = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
    $ret = array('result' => 'ERROR', 'error' => $errors['search']);
    print json_encode($ret);
    exit;
  }
}
?>
