<?php
class ManageUserAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('AJAX', 'validateAjax');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'manageUser';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 25;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Nutzerverwaltung");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL && $user->getIsAdmin() != 'f')
      return true;
    return false;
  }

  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    $user = self::getLoggedUser();
    $users = array();
    foreach($user->getUsersForAdmin() as $u)
      array_push($users, $u->getAccountname());
    if($this->request->getSubmitParameter('delete') != '')
    {
      $definitions = array(
          'delete' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    elseif($this->request->getSubmitParameter('lock') != '')
    {
      $definitions = array(
          'lock'   => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    elseif($this->request->getSubmitParameter('unlock') != '')
    {
      $definitions = array(
          'unlock' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    $names = array(
          'delete' => _('Nutzer zu l&ouml;schen'),
          'lock'   => _('Nutzer zu sperren'),
          'unlock' => _('Nutzer zu entsperren'),
    );
    $errorMsgs = array(
          'delete' => array(myMVC_BaseAction::$ERROR_RANGE     => _('Der Nutzer existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'lock'   => array(myMVC_BaseAction::$ERROR_RANGE     => _('Der Nutzer existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'unlock' => array(myMVC_BaseAction::$ERROR_RANGE     => _('Der Nutzer existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );

    if(!is_array($definitions))
      return true;
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    global $config;

    $user = self::getLoggedUser();
    $domains = array();
    if($user->getIsAdmin() == 't')
      $doms = Domain::callFinder('findAll', array());
    else
      $doms = Domain::getDomainsForUser($user->getAccountname());
    foreach($doms as $d)
      array_push($domains, $d->getDomainname());
    $definitions = array(
        'newAlias'                           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255, 'regexp' => '/^[A-z0-9][\w+.-]*$/'),
        'newDomain'                          => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $domains),
        'accountType'                        => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(Account::getAccountTypes())),
    );
    switch($this->request->getSubmitParameter('accountType'))
    {
      case 'MBOX':
        $definitions['login']                = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255, 'regexp' => '/^[A-z0-9][\w.-]*$/',
                                                        'comparedb' => array('query' => 'SELECT COUNT(*) FROM account WHERE accountname = {checkValue}',
                                                                             'compareType' => '==', 'compareValue' => 0));
        $definitions['passwd']               = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 32, 'minlen' => 5);
        $definitions['canModifyAlias']       = array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false);
        $definitions['canUseFilters']        = array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false);
        $definitions['canSetAutoresponder']  = array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false);
        $definitions['userType']             = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                                      'range' => array_keys(Account::getAdminTypes($user->getIsAdmin())));
      break;
      case 'FORWARD':
        $definitions['forward']              = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255);
      break;
      case 'PIPE':
        $pipes = array();
        foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
          array_push($pipes, $pipe['name']);
        $definitions['pipeName']             = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $pipes);
        $definitions['pipeParameter']        = array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                                    'range' => Account::getParametersForPipe($this->request->getSubmitParameter('pipeName')));
      break;
    }
    $names = array(
        'newAlias'                           => _('Alias'),
        'newDomain'                          => _('Domain'),
        'accountType'                        => _('Kontotyp'),
        'login'                              => _('Benutzername'),
        'passwd'                             => _('Passwort'),
        'canModifyAlias'                     => _('Aliasrechte'),
        'canUseFilters'                      => _('Filterrechte'),
        'canSetAutoresponder'                => _('Autoresponderrechte'),
        'userType'                           => _('Nutzertyp'),
        'forward'                            => _('Weiterleitungsziele'),
        'pipeName'                           => _('Programm'),
        'pipeParameter'                      => _('Parameter'),
    );
    $errorMsgs = array(
        'newAlias'                             => array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_REGEXP    => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'newDomain'                            => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'accountType'                          => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'login'                                => array(myMVC_BaseAction::$ERROR_COMPAREDB => sprintf(_("%s ist bereits vergeben"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_REGEXP    => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'passwd'                               => array(myMVC_BaseAction::$ERROR_MINLEN    => sprintf(_("%s muss zwischen 5 und 32 Zeichen lang sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s muss zwischen 5 und 32 Zeichen lang sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'canModifyAlias'                       => array(myMVC_BaseAction::$ERROR_NOTBOOL   => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'canUseFilters'                        => array(myMVC_BaseAction::$ERROR_NOTBOOL   => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'canSetAutoresponder'                  => array(myMVC_BaseAction::$ERROR_NOTBOOL   => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'userType'                             => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'forward'                              => array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'pipeName'                             => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'pipeParameter'                        => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                                        myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as AJAX.
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateAjax()
  {
    global $config;

    if($this->request->getSubmitParameter('action') == 'getPipeParameters')
    {
      $pipes = array();
      foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
        array_push($pipes, $pipe['name']);
      $definitions = array(
          'pipe'     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $pipes),
      );
    }
    else
    {
      $definitions = array(
          'search'   => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 2, 'maxlen' => 255, 'empty' => true),
      );
    }
    $names = array(
        'search'     => _('Suchkriterium'),
        'pipe'       => _('Programm'),
    );
    $errorMsgs = array(
        'search'     => array(myMVC_BaseAction::$ERROR_NOTSTRING   => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                              myMVC_BaseAction::$ERROR_MINLEN      => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}'),
                              myMVC_BaseAction::$ERROR_MAXLEN      => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}')),
        'pipe'       => array(myMVC_BaseAction::$ERROR_EMPTY       => sprintf(_("%s kann nicht leer sein"), '{field}'),
                              myMVC_BaseAction::$ERROR_RANGE       => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                              myMVC_BaseAction::$ERROR_NOTSTRING   => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();
    $user = self::getLoggedUser();
    
    $response->setNext('ManageUserView');
    $this->session->useNamespace('manageUser');
    
    if($this->request->getRequestMethod() == 'GET')
    {
      if($this->request->getSubmitParameter('delete') != '')
      {
        $acc = Account::findByPK(array('accountname' => $this->request->getParameter('delete')));
        if($acc->getAccountType() == 'MBOX')
        {
          $acc->setToDelete('t');
          $acc->save();
        }
        else
        {
          $acc->delete();
        }
        $this->saveParameterForNextPage('popupMessage', _('Das Konto wurde erfolgreich gel&ouml;scht'));
      }
      elseif($this->request->getSubmitParameter('lock') != '')
      {
        $acc = Account::findByPK(array('accountname' => $this->request->getParameter('lock')));
        $acc->setVirtual('t');
        $acc->save();
        $this->saveParameterForNextPage('popupMessage', _('Das Konto wurde gesperrt'));
      }
      elseif($this->request->getSubmitParameter('unlock') != '')
      {
        $acc = Account::findByPK(array('accountname' => $this->request->getParameter('unlock')));
        $acc->setVirtual('f');
        $acc->save();
        $this->saveParameterForNextPage('popupMessage', _('Das Konto wurde entsperrt'));
      }
      else
      {
        $this->saveParameterForNextPage('canModifyAlias', 't');
        $this->saveParameterForNextPage('canUseFilters', 'f');
        $this->saveParameterForNextPage('canSetAutoresponder', 't');
        $this->saveParameterForNextPage('userType', 'f');
        $this->saveParameterForNextPage('accountType', 'MBOX');
      }
    }
    else
    {
      $info = posix_getpwnam($config['MBOXCONFIG']['EXIMUSER']);
      $na = new Account();
      switch($this->request->getParameter('accountType'))
      {
        case 'MBOX':
          $na->setAccountname($this->request->getParameter('login'));
          $na->setPasswd($this->request->getParameter('passwd'));
          $na->setDataHam('save %%FINALDEST%%');
          $na->setDataSpam(sprintf('pipe "/opt/exim/logSpam.sh $header_from: $header_subject: %s"\nsave %%FINALDESTSPAM%%',
              $this->request->getParameter('login')));
          $na->setFinaldest($na->getMailbox('mail', $this->request->getParameter('newDomain')));
          $na->setFinaldestSpam($na->getMailbox('spam', $this->request->getParameter('newDomain')));
          $na->setVirtual('f');
          $na->setIsAdmin($this->request->getParameter('userType'));
          $na->setCanModifyAlias($this->request->getParameter('canModifyAlias'));
          $na->setCanSetAutoresponder($this->request->getParameter('canSetAutoresponder'));
          $na->setCanUseFilters($this->request->getParameter('canUseFilters'));
          $na->setToActivate('t');
        break;
        case 'FORWARD':
          $na->setAccountname(sprintf('w_%s_%s',
                $this->request->getParameter('newDomain'),
                $this->request->getParameter('newAlias')));
          $na->setPasswd(NULL);
          $fData = '';
          foreach(explode("\n", str_replace("\r\n", "\n", rtrim($this->request->getParameter('forward')))) as $f)
          {
            if($fData != '')
              $fData .= "\n";
            $fData .= "deliver $f";
          }
          $na->setDataHam($fData);
          $na->setDataSpam(NULL);
          $na->setFinaldest(NULL);
          $na->setFinaldestSpam(NULL);
          $na->setVirtual('t');
          $na->setIsAdmin('f');
          $na->setCanModifyAlias('f');
          $na->setCanSetAutoresponder('f');
          $na->setCanUseFilters('f');
          $na->setToActivate('f');
        break;
        case 'PIPE':
          $na->setAccountname(sprintf('p_%s_%s',
                $this->request->getParameter('newDomain'),
                $this->request->getParameter('newAlias')));
          $na->setPasswd(NULL);
          foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
          {
            if($pipe['name'] == $this->request->getParameter('pipeName'))
            {
              list($type, $v) = explode(':', $pipe['values']);
              list($prefix, $other) = explode('|', $v);
              if($other == '' && $prefix != '')
              {
                $other = $prefix;
                $prefix = '';
              }
              $fData = sprintf('pipe "%s %s%s"',
                  $pipe['command'], $prefix, $this->request->getParameter('pipeParameter'));
            }
          }
          $na->setDataHam($fData);
          $na->setDataSpam(NULL);
          $na->setFinaldest(NULL);
          $na->setFinaldestSpam(NULL);
          $na->setVirtual('t');
          $na->setIsAdmin('f');
          $na->setCanModifyAlias('f');
          $na->setCanSetAutoresponder('f');
          $na->setCanUseFilters('f');
          $na->setToActivate('f');
        break;
      }
      $na->setAccountType($this->request->getParameter('accountType'));
      $na->setFilters(NULL);
      $na->setHasAutoresponder('f');
      $na->setAutoresponderSubject(NULL);
      $na->setAutoresponderText(NULL);
      $na->setUid($info['uid']);
      $na->setSpamScanType('SPAMFOLDER');
      $na->setVirusScanType('REJECT');
      $na->setUseSPF('t');
      $na->setUseGreylist('t');
      $na->setUseDomainKey('f');
      $na->setToDelete('f');
      $na->setRejectedSpamScore(0);
      $na->setRejectAfterSize(NULL);
      $na->save();
      $this->saveParameterForNextPage('popupMessage', _('Das Konto wurde hinzugef&uuml;t und ist in K&uuml;rze nutzbar'));

      $nal = new Aliases();
      $nal->setAccountname($na->getAccountname());
      $nal->setDomainname($this->request->getParameter('newDomain'));
      $nal->setAliasname($this->request->getParameter('newAlias'));
      $nal->save();

      $this->saveParameterForNextPage('canModifyAlias', 't');
      $this->saveParameterForNextPage('canUseFilters', 'f');
      $this->saveParameterForNextPage('canSetAutoresponder', 't');
      $this->saveParameterForNextPage('userType', 'f');
    }
    $this->saveParameterForNextPage('accounts', $user->getUsersForAdmin(NULL, true));

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $user = self::getLoggedUser();
    $response = new myMVC_HttpResponse();
    $response->setNext('ManageUserView');
    $this->saveParameterForNextPage('error', true);
    $this->saveParameterForNextPage('accounts', $user->getUsersForAdmin(NULL, true));
    if($this->request->getSubmitParameter('accountType') == 'PIPE')
      $this->saveParameterForNextPage('pipeParameters', Account::getParametersForPipe($this->request->getSubmitParameter('pipeName')));

    return $response;
  }

  /**
   * Perform the AJAX-Action
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function performAjax()
  {
    global $controller, $config;

    $this->session->useNamespace('main');
    header('Content-Type: application/json');
    $user = self::getLoggedUser();

    if($this->request->getSubmitParameter('action') == 'getPipeParameters')
    {
      print json_encode(array('result' => 'OK', 'data' => Account::getParametersForPipe($this->request->getParameter('pipe'))));
    }
    else
    {
      if($this->request->getParameter('search') != '')
        $accounts = $user->getUsersForAdmin($this->request->getParameter('search'), true);
      else
        $accounts = $user->getUsersForAdmin(NULL, true);
  
      $renderEngine = new myMVC_SmartyEngine(dirname(__FILE__).'/../pages/templates',
                                             dirname(__FILE__).'/../pages/templates_c');
      $view = new SearchView($renderEngine, $this->session, 'searchUsers.tpl');
      $view->setObject('accounts', $accounts);
      $controller->setCurrentLanguage();
      if($view->preRender())
        $html = $view->fetch();
      else
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SearchView::preRender() returned false. Unable to display page');
        throw new myMVC_MVCException($msg);
      }
      print json_encode(array('result' => 'OK', 'html' => $html));
    }

    exit;
  }

  /**
   * Called if user data are not valid (check with validate)
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidDataAjax()
  {
    header('Content-Type: application/json');
    $errors = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
    $ret = array('result' => 'ERROR', 'error' => $errors['search']);
    print json_encode($ret);
    exit;
  }
}
?>
